package org.jboss.security.jacc;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.lang.reflect.Constructor;
import java.security.AccessController;
import java.security.Policy;
import java.security.PrivilegedAction;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.security.jacc.PolicyContext;
import javax.security.jacc.PolicyContextHandler;
import org.jboss.logging.Logger;
import org.jboss.security.jacc.SubjectPolicyContextHandler;

public class SecurityService {
  private static final String JACC_POLICY_PROVIDER = "javax.security.jacc.policy.provider";
  
  private static final Logger log = Logger.getLogger(org.jboss.security.jacc.SecurityService.class);
  
  private Policy oldPolicy;
  
  private Policy jaccPolicy;
  
  private ObjectName policyName;
  
  private String policyAttributeName = "Policy";
  
  private MBeanServer server;
  
  public ObjectName getPolicyName() {
    return this.policyName;
  }
  
  public void setPolicyName(ObjectName policyName) {
    this.policyName = policyName;
  }
  
  public String getPolicyAttributeName() {
    return this.policyAttributeName;
  }
  
  public void setPolicyAttributeName(String policyAttributeName) {
    this.policyAttributeName = policyAttributeName;
  }
  
  public MBeanServer getMBeanServer() {
    return this.server;
  }
  
  public void setMBeanServer(MBeanServer server) {
    this.server = server;
  }
  
  public void start() throws Exception {
    this.oldPolicy = Policy.getPolicy();
    if (this.server != null && this.policyName != null && this.server.isRegistered(this.policyName))
      try {
        this.jaccPolicy = (Policy)this.server.getAttribute(this.policyName, this.policyAttributeName);
      } catch (Exception e) {
        log.warn("Failed to get " + this.policyAttributeName + " attribute from: " + this.policyName, e);
      }  
    if (this.jaccPolicy == null) {
      String provider = getProperty("javax.security.jacc.policy.provider", "org.jboss.security.jacc.DelegatingPolicy");
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      Class providerClass = loader.loadClass(provider);
      try {
        Class[] ctorSig = { Policy.class };
        Constructor ctor = providerClass.getConstructor(ctorSig);
        Object[] ctorArgs = { this.oldPolicy };
        this.jaccPolicy = (Policy)ctor.newInstance(ctorArgs);
      } catch (NoSuchMethodException e) {
        log.debug("Provider does not support ctor(Policy)");
        this.jaccPolicy = (Policy)providerClass.newInstance();
      } 
    } 
    Policy.setPolicy(this.jaccPolicy);
    this.jaccPolicy.refresh();
    SubjectPolicyContextHandler handler = new SubjectPolicyContextHandler();
    PolicyContext.registerHandler("javax.security.auth.Subject.container", (PolicyContextHandler)handler, true);
  }
  
  public void stop() throws Exception {
    if (this.jaccPolicy != null)
      Policy.setPolicy(this.oldPolicy); 
  }
  
  static String getProperty(String name) {
    return getProperty(name, null);
  }
  
  static String getProperty(String name, String defaultValue) {
    PropertyAccessAction propertyAccessAction = new PropertyAccessAction(name, defaultValue);
    String property = AccessController.<String>doPrivileged((PrivilegedAction)propertyAccessAction);
    return property;
  }
}
